<?php

namespace App\Http\Controllers\Api\V1;

use App\Http\Controllers\Controller;
use App\Http\Requests\RegisterRequest;
use App\Mail\ForgetPassword;
use App\Models\OwnerApproval;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Str;

class AuthController extends Controller
{
    public function register(RegisterRequest $request) {
        $user = User::query()->create($request->validated());


        if ($user->role == "user") {
            // create an owner approval request
            OwnerApproval::query()->create([
                "owner_id" => $user->owner_id,
                "user_id" => $user->id,
            ]);

            return response([
                "success" => true,
                "message" => "A request has been sent to the owner of the flat to access the app. Once the request is approved by the owner, you can login with your credentials!"
            ]);
        }

        return response([
            "success" => true,
            "message" => "Registered successfully!"
        ], 201);
    }

    public function login(Request $request) {
        $request->validate([
            "email" => "sometimes|email",
            'password' => 'required|string'
        ]);

        // Check email
        if ($request->has('email')) {
            $user = User::where('email', $request->email)->first();
        } elseif ($request->has('phone')) {
            $user = User::where('phone', $request->phone)->first();
        } else {
            return response([
                "success" => false,
                'message' => 'Bad credentials'
            ], 401);
        }

        // Check password
        if (!$user || !Hash::check($request->password, $user->password)) {
            return response([
                "success" => false,
                'message' => 'Bad credentials'
            ], 401);
        }

        // check if the user is not approved by owner of the flat
        if ($user->role == "user" && !($user->is_approved)) {
            return response([
                "success" => false,
                "message" => "Your request is not approved by owner of the flat yet!, Please contact the owner"
            ], 401);
        }

        // delete previous tokens and generate a new token
        $user->tokens()->delete();
        $token = $user->createToken('app_token')->plainTextToken;

        // load relations if the user is not admin
        if ($user->role !== "admin") {
            $user->loadMissing("flat.society", "flat.wing", "flat.owner");
        }

        return response([
            "user" => $user,
            "token" => $token
        ]);
    }


    public function logout(Request $request) {
        auth()->user()->tokens()->delete();

        return [
            'message' => 'Logged out'
        ];
    }

    public function forgetPassword(Request $request) {
        $fields = Validator::make($request->all(), [
            'email' => 'required|string|email',
        ]);

        if ($fields->fails()) {
            return response()->json(['error' => $fields->errors()], 422);
        }

        $user = User::where('email', $request->email)->first();

        if (!isset($user)) {
            return response()->json([
                "status" => "failed",
                "status_message" => "User not found"
            ], 404);
        }

        $user->forget_password_token = Str::random(32);
        Mail::to($user->email)->send(new ForgetPassword($user));
        $user->save();

        return response([
            'status' => 'success',
            'status_message' => 'Password reset link has been sent to your registered email id. Click on it to reset your password.'
        ]);
    }

    public function updatePassword(Request $request) {
        $fields = Validator::make($request->all(), [
            'token' => 'required|string',
            'password' => 'required|string',
            'confirmation_password' => 'required|string|same:password'
        ]);

        if ($fields->fails()) {
            return response()->json(['error' => $fields->errors()], 422);
        }

        $user = User::where('forget_password_token', $request->token)->first();

        if (!$user) {
            return response()->json([
                'error' => 'User does not exists or token expired'
            ], 401);
        }

        $user->password = $request->password;
        $user->save();
        return response()->json(['success' => 'Password changed successfully']);
    }

    public function googleLogin(Request $request) {
        // validate request body
        $fields = Validator::make($request->all(), [
            'email' => 'required|string|email|unique:users,email'
        ]);

        if ($fields->fails()) {
            return response()->json([
                'status_message' => $fields->errors()->first()
            ], 422);
        }

        // check if user already exists
        $oldUser = User::where('email', $request->email)->first();
        if (isset($oldUser)) {
            $token = $oldUser->createToken('app_token')->plainTextToken;

            return response([
                "user" => $oldUser,
                "token" => $token
            ], 200);
        }

        // create new user
        $user = User::create([
            'email' => $request->email,
            'password' => bcrypt('password'),
            'is_google_login' => true
        ]);
        $token = $user->createToken('app_token')->plainTextToken;

        return response([
            "user" => $user,
            "token" => $token
        ], 200);
    }
}
